/*****************************************************************************************************************
Written by Artur Araujo
April-June 2016
Modified by Artur Araujo
December 2017

artur.stat@gmail.com

while working for the
Luxembourg Institute of Health
on the IDEAL project

http://www.ideal.rwth-aachen.de/
*****************************************************************************************************************/

* Log current folder ;
%xlog(cd);

* Determine path to script ;
%let ScriptDir=%sysfunc(
	tranwrd(
		%sysget(SAS_EXECFILEPATH),
		%sysget(SAS_EXECFILENAME),
	)
);

* Set current folder ;
x cd "&ScriptDir";

* Log current folder ;
%xlog(cd);

* Define local libraries ;
libname NORMAL "..\library\normal";

* Define file reference ;
filename nof1 DISK '..\data\N of 1 example.csv';

/*****************************************************************************************************************
Programs for
Araujo, Artur, 2018, Statistical Analysis of Series of N-of-1 Trials Using SAS

Chapter 5: Importing and handling data using SQL
*****************************************************************************************************************/

/*****************************************************************************************************************
Program 5.1: Importing data from an external file using proc import
*****************************************************************************************************************/

proc import
		datafile=nof1
		out=WORK.ndata(
			rename=(VAR1=Patient VAR2=Treatment VAR3=Cycle VAR4=Pair VAR5=Y)
		)
		dbms=dlm
		replace;
	datarow=2;
	delimiter=",";
	getnames=no;
run;

/*****************************************************************************************************************
Program 5.2: Adding period information to the data using SQL
*****************************************************************************************************************/

* Determine and count unique Patients ;
proc sql noprint;
	select	distinct Patient
			into :Patient1 - :Patient999999999
	from WORK.ndata;
quit;
%let PatientNumber = &SqlObs;

* Declare %sqlBody212 macro ;
%macro sqlBody212;
	%local i;
	%do i=2 %to &PatientNumber %by 1;
		union
		select	Patient,
	 			Treatment,
				Cycle,
				monotonic() as Period,
				Y
		from WORK.ndata
		where Patient eq &&Patient&i
	%end;
%mend sqlBody212;

* Determine Period variable from WORK.ndata,
save the resulting data to WORK.periodata,
and finally delete WORK.ndata;
proc sql;
	create table WORK.periodata as
	select	Patient,
			Treatment,
			Cycle,
			monotonic() as Period,
			Y
	from WORK.ndata
	where Patient eq &Patient1
	%sqlBody212;
	drop table WORK.ndata;
quit;

/*****************************************************************************************************************
Program 5.3: Removing global macro variables from the SAS system
*****************************************************************************************************************/

* Declare %DeleteGlobal macro ;
%macro DeleteGlobal;
	%local i;
	%do i=1 %to &PatientNumber %by 1;
		%symdel Patient&i;
	%end;
	%symdel PatientNumber;
%mend DeleteGlobal;

* Delete global macro variables ;
%DeleteGlobal

* Delete macros from WORK.Sasmacr ;
proc catalog cat=WORK.Sasmacr;
	delete sqlBody212 DeleteGlobal / et=macro;
quit;

/*****************************************************************************************************************
Program 5.4: Converting data types and sorting the data using SQL
*****************************************************************************************************************/

proc sql;
	create table NORMAL.ndata as
	select	Patient,
			Treatment,
			left( put(Cycle, 8.) ) as Cycle,
			left( put(Period, 8.) ) as Period,
			Y
	from WORK.periodata
	order by Patient, Period;
	drop table WORK.periodata;
quit;

/*****************************************************************************************************************
Program 5.5: Creating unbalanced data using SQL
*****************************************************************************************************************/

proc sql;
	create table NORMAL.ndata_unbalanced as
	select *
	from NORMAL.ndata
	where not (Patient eq 11 and Cycle eq "3")
	and not (Patient eq 12 and Cycle ne "1");
quit;

/*****************************************************************************************************************
Program 5.6: Creating a dataset of outcome differences using SQL
*****************************************************************************************************************/

proc sql;
	create view A as
	select	Patient,
			Cycle,
			Y as YA
	from NORMAL.ndata
	where Treatment eq "A";
	create view B as
	select	Patient,
			Cycle,
			Y as YB
	from NORMAL.ndata
	where Treatment eq "B";
	create table NORMAL.ddata as
	select	A.Patient,
			A.Cycle,
			YA,
			YB,
			YB-YA as dY
	from A join B
	on A.Patient eq B.Patient
	and A.Cycle eq B.Cycle
	order by Patient, Cycle;
	drop view A, B;
quit;

/*****************************************************************************************************************
Program 5.7: Creating an unbalanced dataset of outcome differences using SQL
*****************************************************************************************************************/

proc sql;
	create table NORMAL.ddata_unbalanced as
	select *
	from NORMAL.ddata;
	delete from NORMAL.ddata_unbalanced
	where (Patient eq 11 and Cycle eq "3")
	or (Patient eq 12 and Cycle ne "1");
quit;
