/*****************************************************************************************************************
Written by Artur Araujo
April-June 2016

artur.stat@gmail.com

while working for the
Luxembourg Institute of Health
on the IDEAL project

http://www.ideal.rwth-aachen.de/
*****************************************************************************************************************/

* Log current folder ;
%xlog(cd);

* Determine path to script ;
%let ScriptDir=%sysfunc(
	tranwrd(
		%sysget(SAS_EXECFILEPATH),
		%sysget(SAS_EXECFILENAME),
	)
);

* Set current folder ;
x cd "&ScriptDir";

* Log current folder ;
%xlog(cd);

* Define local libraries ;
libname NORMAL "..\library\normal";

* Define local macros folder ;
filename MACROS '..\macros';

* Include macros ;
%include MACROS('*.sas');

/*****************************************************************************************************************
Programs for
Araujo, Artur, 2018, Statistical Analysis of Series of N-of-1 Trials Using SAS

Chapter 12: Linear fixed-effects model of difference
*****************************************************************************************************************/

/*****************************************************************************************************************
Subchapter 12.1: Estimation and inference on the overall treatment effect
*****************************************************************************************************************/

/*****************************************************************************************************************
Program 12.1: Fitting a linear fixed-effects model of difference using proc glm- Balanced data
*****************************************************************************************************************/

title "Balanced data";
proc glm
		data=NORMAL.ddata
		alpha=0.05 /* level for CI */
		plots=none; * suppress plots ;
	class Patient;
	model dY = Patient /
		alpha=0.05 /* level for CI */
		clparm; * CI for parameter estimates ;
	estimate 'Overall mean'
		intercept			1;
run;
quit;

/*****************************************************************************************************************
Program 12.2: Fitting a linear fixed-effects model of difference using proc glm- Unbalanced data
*****************************************************************************************************************/

title "Unbalanced data";
proc glm
		data=NORMAL.ddata_unbalanced
		alpha=0.05 /* level for CI */
		plots=none; * suppress plots ;
	class Patient;
	model dY = Patient /
		alpha=0.05 /* level for CI */
		clparm; * CI for parameter estimates ;
	estimate 'Overall mean'
		intercept			1;
run;
quit;

/*****************************************************************************************************************
Program 12.3: Determining the number of observations
*****************************************************************************************************************/

title "Unbalanced data";
title2 "Total number of observations";
proc sql;
	select	count(dY) as nobs
	from NORMAL.ddata_unbalanced;
quit;

/*****************************************************************************************************************
Program 12.4: Determining the number of observations per patient
*****************************************************************************************************************/

title "Unbalanced data";
title2 "Number of observations per Patient";
proc sql;
	select	Patient,
			count(dY) as nobs
	from NORMAL.ddata_unbalanced
	group by Patient;
quit;

/*****************************************************************************************************************
Program 12.5: Fitting a linear fixed-effects model of difference using proc glm- Weighted estimate
*****************************************************************************************************************/

title "Unbalanced data";
title2 "Weighted estimate";
proc glm
		data=NORMAL.ddata_unbalanced
		alpha=0.05 /* level for CI */
		plots=none; * suppress plots ;
	class Patient;
	model dY = Patient /
		alpha=0.05 /* level for CI */
		clparm /* CI for parameter estimates */
		ss1
		ss2
		ss3;
	estimate 'Overall mean'
		intercept			 33
		Patient				 3  3  3  3  3  3  3  3  3  3  2  1 /
		divisor=33;
run;
quit;

/*****************************************************************************************************************
Subchapter 12.2: Estimation and inference on individual treatment effects
*****************************************************************************************************************/

/*****************************************************************************************************************
Program 12.6: Estimating individual treatment effects using 'proc glm' - The hard way
*****************************************************************************************************************/

title "Individual treatment effects";
title2 "Balanced data";
proc glm
		data=NORMAL.ddata
		alpha=0.05 /* level for CI */
		plots=none; * suppress plots ;
	class Patient;
	model dY = Patient /
		alpha=0.05 /* level for CI */
		clparm; * CI for parameter estimates ;
	estimate '1' intercept 1
		Patient	1;
	estimate '2' intercept 1
		Patient 0 1;
	estimate '3' intercept 1
		Patient 0 0 1;
	estimate '4' intercept 1
		Patient 0 0 0 1;
	estimate '5' intercept 1
		Patient 0 0 0 0 1;
	estimate '6' intercept 1
		Patient 0 0 0 0 0 1;
	estimate '7' intercept 1
		Patient 0 0 0 0 0 0 1;
	estimate '8' intercept 1
		Patient 0 0 0 0 0 0 0 1;
	estimate '9' intercept 1
		Patient 0 0 0 0 0 0 0 0 1;
	estimate '10' intercept 1
		Patient 0 0 0 0 0 0 0 0 0 1;
	estimate '11' intercept 1
		Patient 0 0 0 0 0 0 0 0 0 0 1;
	estimate '12' intercept 1
		Patient 0 0 0 0 0 0 0 0 0 0 0 1;
run;
quit;

/*****************************************************************************************************************
Program 12.7: Estimating individual treatment effects using proc glm - The macro way
*****************************************************************************************************************/

* Write global macro variable array ;
%estimate_diff_global(
	array=vdiff_unbalanced,
	subject=Patient,
	data=NORMAL.ddata_unbalanced
)

title "Individual treatment effects";
title2 "Unbalanced data";
proc glm
		data=NORMAL.ddata_unbalanced
		alpha=0.05 /* level for CI */
		plots=none; * suppress plots ;
	class Patient;
	model dY = Patient /
		alpha=0.05 /* level for CI */
		clparm; * CI for parameter estimates ;
	%estimate_diff(array=vdiff_unbalanced, random=)
run;
quit;

* Delete global macro variable array ;
%estimate_diff_free(array=vdiff_unbalanced)

* Log global macro variables ;
%put _GLOBAL_;

title; * clear all titles ;
