/*****************************************************************************************************************
Written by Artur Araujo
April-June 2016
Modified by Artur Araujo
January 2017-April 2018

artur.stat@gmail.com

while working for the
Luxembourg Institute of Health
on the IDEAL project

http://www.ideal.rwth-aachen.de/
*****************************************************************************************************************/

* Log current folder ;
%xlog(cd);

* Determine path to script ;
%let ScriptDir=%sysfunc(
	tranwrd(
		%sysget(SAS_EXECFILEPATH),
		%sysget(SAS_EXECFILENAME),
	)
);

* Set current folder ;
x cd "&ScriptDir";

* Log current folder ;
%xlog(cd);

* Define local libraries ;
libname NORMAL "..\library\normal";

* Define local macros folder ;
filename MACROS '..\macros';

* Include macros ;
%include MACROS('*.sas');

/*****************************************************************************************************************
Programs for
Araujo, Artur, 2018, Statistical Analysis of Series of N-of-1 Trials Using SAS

Chapter 14: Linear mixed-effects model of difference
*****************************************************************************************************************/

/*****************************************************************************************************************
Subchapter 14.1: Estimation and inference on the overall treatment effect
*****************************************************************************************************************/

/*****************************************************************************************************************
Program 14.1: Fitting a linear mixed-effects model of difference using proc mixed  Balanced data
*****************************************************************************************************************/

title "Balanced data";
proc mixed
		data=NORMAL.ddata
		alpha=0.05 /* level for CI */
		cl /* CI for covariance parameters */
		method=REML; * REML method for estimation ;
	class Patient;
	model dY = /
		alpha=0.05 /* level for CI */
		cl /* t-type CI for fixed effects */
		covb /* covariance matrix of fixed effects estimates */
		ddfm=KR /* inference based on Kenward and Roger's method */
		htype=2 /* type 2 hypothesis test */
		solution; * solution for fixed effects ;
	random Patient;
run;

/*****************************************************************************************************************
Program 14.2: Fitting a linear mixed-effects model of difference using proc mixed - Unbalanced data
*****************************************************************************************************************/

title "Unbalanced data";
proc mixed
		data=NORMAL.ddata_unbalanced
		alpha=0.05 /* level for CI */
		cl /* CI for covariance parameters */
		method=REML; * REML method for estimation ;
	class Patient;
	model dY = /
		alpha=0.05 /* level for CI */
		cl /* t-type CI for fixed effects */
		covb /* covariance matrix of fixed effects estimates */
		ddfm=KR /* inference based on Kenward and Roger's method */
		htype=2 /* type 2 hypothesis test */
		solution; * solution for fixed effects ;
	random Patient;
run;

/*****************************************************************************************************************
Subchapter 14.2: Estimation and inference on individual treatment effects
*****************************************************************************************************************/

/*****************************************************************************************************************
Program 14.3: Estimating individual treatment effects using proc mixed  Kenward and Rogers inference
*****************************************************************************************************************/

title "Shrunk individual treatment effects";
title2 "Balanced data";
title3 "Kenward-Roger";
proc mixed
		data=NORMAL.ddata
		alpha=0.05 /* level for CI */
		cl /* CI for covariance parameters */
		method=REML; * REML method for estimation ;
	class Patient;
	model dY = / ddfm=KR;
	random Patient;
	estimate '1' intercept 1 |
		Patient 1 /
			alpha=0.05 /* level for CI */
			cl; * t-type CI for estimates ;
	estimate '2' intercept 1 |
		Patient 0 1;
	estimate '3' intercept 1 |
		Patient 0 0 1;
	estimate '4' intercept 1 |
		Patient 0 0 0 1;
	estimate '5' intercept 1 |
		Patient 0 0 0 0 1;
	estimate '6' intercept 1 |
		Patient 0 0 0 0 0 1;
	estimate '7' intercept 1 |
		Patient 0 0 0 0 0 0 1;
	estimate '8' intercept 1 |
		Patient 0 0 0 0 0 0 0 1;
	estimate '9' intercept 1 |
		Patient 0 0 0 0 0 0 0 0 1;
	estimate '10' intercept 1 |
		Patient 0 0 0 0 0 0 0 0 0 1;
	estimate '11' intercept 1 |
		Patient 0 0 0 0 0 0 0 0 0 0 1;
	estimate '12' intercept 1 |
		Patient 0 0 0 0 0 0 0 0 0 0 0 1;
run;

/*****************************************************************************************************************
Program 14.4: Estimating individual treatment effects using proc mixed and asymptotic inference  The macro way
*****************************************************************************************************************/

* Write global macro variable array ;
%estimate_diff_global(
	array=vdiff,
	subject=Patient,
	data=NORMAL.ddata
)

title "Shrunk individual treatment effects";
title2 "Balanced data";
proc mixed
		data=NORMAL.ddata
		alpha=0.05 /* level for CI */
		cl /* CI for covariance parameters */
		method=REML; * REML method for estimation ;
	class Patient;
	model dY = ;
	random Patient;
	* insert estimate statements ;
	%estimate_diff(array=vdiff, options=alpha=0.05 cl)
run;

* Delete global macro variable array ;
%estimate_diff_free(array=vdiff)

/*****************************************************************************************************************
Program 14.5: Estimating individual treatment effects using proc mixed and KR inference The macro way
*****************************************************************************************************************/

* Write global macro variable array ;
%estimate_diff_global(
	array=vdiff_unbalanced,
	subject=Patient,
	data=NORMAL.ddata_unbalanced
)

title "Shrunk individual treatment effects";
title2 "Unbalanced data";
title3 "Kenward-Roger";
proc mixed
		data=NORMAL.ddata_unbalanced
		alpha=0.05 /* level for CI */
		cl /* CI for covariance parameters */
		method=REML; * REML method for estimation ;
	class Patient;
	model dY = / ddfm=KR;
	random Patient;
	* insert estimate statements ;
	%estimate_diff(array=vdiff_unbalanced, options=alpha=0.05 cl)
run;

* Delete global macro variable array ;
%estimate_diff_free(array=vdiff_unbalanced)

/*****************************************************************************************************************
Program 14.6: Estimating individual treatment effects using proc mixed - Predicted values
*****************************************************************************************************************/

title "Shrunk individual treatment effects";
title2 "Unbalanced data";
proc mixed
		data=NORMAL.ddata_unbalanced
		alpha=0.05 /* level for CI */
		cl /* CI for covariance parameters */
		method=REML; * REML method for estimation ;
	class Patient;
	model dY = /
		outp=WORK.pddata(drop=Resid); * output dataset for predicted values;
	id Patient; * variables from input dataset in outp= dataset;
	random Patient;
run;

title "Shrunk individual treatment effect estimates";
proc sql;
	select	distinct *
	from WORK.pddata;
	drop table WORK.pddata;
quit;

/*****************************************************************************************************************
Program 14.7: Estimating individual treatment effects using 'proc mixed' - The alternative way
*****************************************************************************************************************/

title "Shrunk individual treatment effects";
title2 "Balanced data";
proc mixed
		data=NORMAL.ddata
		alpha=0.05 /* level for CI */
		cl /* CI for covariance parameters */
		method=REML; * REML method for estimation ;
	class Patient;
	model dY = /
		alpha=0.05 /* level for CI */
		cl /* t-type CI for fixed effects */
		solution; * solution for fixed effects ;
	random intercept /
		subject=Patient;
	estimate '1' intercept 1 | intercept 1 /
		subject 1
		alpha=0.05 /* level for CI */
		cl; * t-type CI for estimates ;
	estimate '2' intercept 1 | intercept  1 /
		subject 0 1;
	estimate '3' intercept 1 | intercept  1 /
		subject 0 0 1;
	estimate '4' intercept 1 | intercept  1 /
		subject 0 0 0 1;
	estimate '5' intercept 1 | intercept  1 /
		subject 0 0 0 0 1;
	estimate '6' intercept 1 | intercept  1 /
		subject 0 0 0 0 0 1;
	estimate '7' intercept 1 | intercept  1 /
		subject 0 0 0 0 0 0 1;
	estimate '8' intercept 1 | intercept  1 /
		subject 0 0 0 0 0 0 0 1;
	estimate '9' intercept 1 | intercept  1 /
		subject 0 0 0 0 0 0 0 0 1;
	estimate '10' intercept 1 | intercept  1 /
		subject 0 0 0 0 0 0 0 0 0 1;
	estimate '11' intercept 1 | intercept  1 /
		subject 0 0 0 0 0 0 0 0 0 0 1;
	estimate '12' intercept 1 | intercept  1 /
		subject 0 0 0 0 0 0 0 0 0 0 0 1;
run;

/*****************************************************************************************************************
Program 14.8: Estimating individual treatment effects using 'proc mixed' - The alternative macro way
*****************************************************************************************************************/

* Write global macro variable array ;
%estimate_diff_random_global(
	array=vdiff,
	subject=Patient,
	data=NORMAL.ddata
)

title "Shrunk individual treatment effects";
title2 "Balanced data";
proc mixed
		data=NORMAL.ddata
		alpha=0.05 /* level for CI */
		cl /* CI for covariance parameters */
		method=REML; * REML method for estimation ;
	class Patient;
	model dY = /
		alpha=0.05 /* level for CI */
		cl /* t-type CI for fixed effects */
		solution; * solution for fixed effects ;
	random intercept /
		subject=Patient;
	* insert estimate statements ;
	%estimate_diff_random(array=vdiff, options=alpha=0.05 cl)
run;

* Delete global macro variable array ;
%estimate_diff_random_free(array=vdiff)

* Log global macro variables ;
%put _GLOBAL_;

title; * clear all titles ;
