/*****************************************************************************************************************
Written by Artur Araujo
December 2017

artur.stat@gmail.com

while working for the
Luxembourg Institute of Health
on the IDEAL project

http://www.ideal.rwth-aachen.de/
*****************************************************************************************************************/

* Log current folder ;
%xlog(cd);

* Determine path to script ;
%let ScriptDir=%sysfunc(
	tranwrd(
		%sysget(SAS_EXECFILEPATH),
		%sysget(SAS_EXECFILENAME),
	)
);

* Set current folder ;
x cd "&ScriptDir";

* Log current folder ;
%xlog(cd);

* Define local libraries ;
libname NORMAL "..\library\normal";

* Define local macros folder ;
filename MACROS '..\macros';

* Include macros ;
%include MACROS('*.sas');

/*****************************************************************************************************************
Programs for
Araujo, Artur, 2018, Statistical Analysis of Series of N-of-1 Trials Using SAS

Chapter 16: Modelling time dependency
*****************************************************************************************************************/

/*****************************************************************************************************************
Program 16.1: Fitting a first-order autorregressive linear mixed-effects model with proc mixed
*****************************************************************************************************************/

* Write global macro variable array ;
%estimate_full_global(
	array=vfull_balanced,
	subject=Patient,
	treatment=Treatment,
	data=NORMAL.ndata
)

title "First-order autoregressive linear mixed-effects model";
proc mixed
		data=NORMAL.ndata
		alpha=0.05 /* level for CI */
		cl /* CI for covariance parameters */
		method=REML; * REML method for estimation ;
	class Treatment (ref=first) Patient Period;
	model Y = Treatment /
		alpha=0.05 /* level for CI */
		cl /* t-type CI for fixed effects */
		htype=2 /* type 2 hypothesis test */
		solution; * solution for fixed effects ;
	random Patient Patient*Treatment;
	repeated Period /
		subject=Patient
		type=ar(1)
		r
		rcorr;
	%estimate_full(array=vfull_balanced, options=alpha=0.05 cl)
run;

/*****************************************************************************************************************
Program 16.2: Fitting a first-order autorregressive moving-average linear mixed-effects model with proc mixed
*****************************************************************************************************************/

title "First-order autoregressive moving-average linear mixed-effects model";
proc mixed
		data=NORMAL.ndata
		alpha=0.05 /* level for CI */
		cl /* CI for covariance parameters */
		method=REML; * REML method for estimation ;
	class Treatment (ref=first) Patient Period;
	model Y = Treatment /
		alpha=0.05 /* level for CI */
		cl /* t-type CI for fixed effects */
		htype=2 /* type 2 hypothesis test */
		solution; * solution for fixed effects ;
	random Patient Patient*Treatment;
	repeated Period /
		subject=Patient
		type=arma(1,1)
		r
		rcorr;
	%estimate_full(array=vfull_balanced, options=alpha=0.05 cl)
run;

* Delete global macro variable array ;
%estimate_full_free(array=vfull_balanced)

/*****************************************************************************************************************
Program 16.3: Simulating randomized series of n-of-1 trials
*****************************************************************************************************************/

%let seed=314159;

proc iml;
	varNames={'Patient' 'Treatment' 'Cycle' 'Period' 'Y'};
	use NORMAL.ndata;
	read all var varNames; * read variables into vectors ;
	close NORMAL.ndata;
	uPatient=unique(Patient); * get unique patients ;
	uCycle=unique(Cycle); * get unique cycles ;
	call randseed(&seed); * set seed for RNG;
	do i=1 to ncol(uPatient) by 1; * loop through the patients ;
		do j=1 to ncol(uCycle) by 1; * loop through the cycles ;
			index=loc(Patient=uPatient[i] & Cycle=uCycle[j]); * locate patient and cycle ;
			call randgen(x, "bernoulli", 0.5); * toss the coin ;
			if x then do; * if true then switch order of treatments ;
				p1=Period[index[1]];
				Period[index[1]]=Period[index[2]];
				Period[index[2]]=p1;
			end;
		end;
	end;
	create WORK.rndata var varNames;
	append;
	close WORK.rndata;
quit;

%symdel seed;

proc sort data=WORK.rndata;
	by Patient Period;
quit;

/*****************************************************************************************************************
Program 16.4: Fitting a first-order autoregressive moving-average linear mixed-effects model with proc mixed -
 Simulated randomized series of n-of-1 trials
*****************************************************************************************************************/

title "First-order autoregressive moving-average linear mixed-effects model";
title2 "Randomized series of n-of-1 trials";

* Write global macro variable array ;
%estimate_full_global(
	array=vfull_rndata,
	subject=Patient,
	treatment=Treatment,
	data=WORK.rndata
)

proc mixed
		data=WORK.rndata
		alpha=0.05 /* level for CI */
		cl /* CI for covariance parameters */
		method=REML; * REML method for estimation ;
	class Treatment (ref=first) Patient Period;
	model Y = Treatment /
		alpha=0.05 /* level for CI */
		cl /* t-type CI for fixed effects */
		htype=2 /* type 2 hypothesis test */
		solution; * solution for fixed effects ;
	random Patient Patient*Treatment;
	repeated Period /
		subject=Patient
		type=arma(1,1)
		r
		rcorr;
	%estimate_full(array=vfull_rndata, options=alpha=0.05 cl)
run;

* Delete global macro variable array ;
%estimate_full_free(array=vfull_rndata)

proc datasets lib=WORK nolist;
	delete rndata;
quit;

* Log global macro variables ;
%put _GLOBAL_;

title; * clear all titles ;
